"""Examples"""
#  AGPL-3.0 license
#  Copyright (c) 2025 Asger Jon Vistisen
from __future__ import annotations

from .type_sig_example import main as typeSigExample

__all__ = [
    "typeSigExample",
]
